package jp.co.sds_corp.util.jmx.naming;

import java.io.InputStream;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.StringRefAddr;

import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.jmx.AbstractMBeanService;

import org.jnp.server.Main;

public class NamingService extends AbstractMBeanService
	implements NamingServiceMBean
{
	Main naming;

	public NamingService() {
		String categoryName = NamingService.class.getName();
		naming = new Main(categoryName);
	}

	public void setPort(int port) {
		naming.setPort(port);
	}

	public int getPort() {
		return naming.getPort();
	}

	public void setRmiPort(int port) {
		naming.setRmiPort(port);
	}

	public int getRmiPort() {
		return naming.getRmiPort();
	}

	public String getBindAddress() {
		return naming.getBindAddress();
	}
	public void setBindAddress(String host) throws UnknownHostException {
		naming.setBindAddress(host);
	}

	public int getBacklog() {
		return naming.getBacklog();
	}
	public void setBacklog(int backlog) {
		naming.setBacklog(backlog);
	}

	public String getClientSocketFactory() {
		return naming.getClientSocketFactory();
	}
	public void setClientSocketFactory(String factoryClassName)
		throws ClassNotFoundException, InstantiationException,
				IllegalAccessException
	{
		naming.setClientSocketFactory(factoryClassName);
	}

	public String getServerSocketFactory() {
		return naming.getServerSocketFactory();
	}
	public void setServerSocketFactory(String factoryClassName)
		throws ClassNotFoundException, InstantiationException,
				IllegalAccessException
	{
		naming.setServerSocketFactory(factoryClassName);
	}

	public ObjectName getObjectName(MBeanServer server, ObjectName name)
		throws javax.management.MalformedObjectNameException
	{
		return new ObjectName(OBJECT_NAME);
	}

	public String getName() {
		return "Naming";
	}

	public void initService( IConfiguration config ) throws Exception {
		if ( config == null ) {
			try {
				ClassLoader loader = Thread.currentThread().getContextClassLoader();
				InputStream is = loader.getResourceAsStream("jndi.properties");
				Properties props = new Properties();
				props.load(is);

				Enumeration keys = props.propertyNames();
				while( keys.hasMoreElements() ) {
					Object obj = keys.nextElement();
					if ( obj == null ) {
						continue;
					}
					String key = (String)obj;
					String value = props.getProperty(key);
					CmnLog.debug(this,"System.setProperty, key="+key+", value="+value);
					System.setProperty(key, value);
				}
			} catch ( Exception ex ) {
				CmnLog.error( this, "naming init exception", ex );
				throw( ex );
			} catch ( Throwable er ) {
				CmnLog.error( this, "naming init error", er );
			}
		} else {
			Iterator itr = config.getChilds( "attribute" );
			while ( itr.hasNext()) {
				IConfiguration c = (IConfiguration )itr.next();
				String key = c.getAttributes().get( "key" );
				String value = c.getAttributes().get( "value" );
				CmnLog.debug( this, "System.setProperty"
							+ " key=" + key + ", value=" + value );
				System.setProperty(key, value);
			}
		}
	}

	public void startService() throws Exception {
		try {
			naming.start();
			InitialContext iniCtx = new InitialContext();
			Hashtable env = iniCtx.getEnvironment();
			Enumeration keys = env.keys();
			CmnLog.debug(this,"InitialContext Environment:");
			String providerURL = null;
			while( keys.hasMoreElements() ) {
				String key = (String) keys.nextElement();
				String value = (String) env.get(key);
				CmnLog.debug(this,"key="+key+", value="+value);
				if( key.equals(Context.PROVIDER_URL) ) {
					providerURL = value;
				}
			}
			if( providerURL != null ) {
				CmnLog.warn(this,"Saw Context.PROVIDER_URL in server jndi.properties, url="+providerURL);
			}

			ClassLoader topLoader = Thread.currentThread().getContextClassLoader();
			ENCFactory.setTopClassLoader(topLoader);
			RefAddr refAddr = new StringRefAddr("nns", "ENC");
			Reference envRef = new Reference("javax.naming.Context", refAddr, ENCFactory.class.getName(), null);
			Context ctx = (Context) iniCtx.lookup("java:");
			ctx.rebind("comp", envRef);
			CmnLog.debug(this,"Naming started on port "+naming.getPort());
		} catch ( Exception ex ) {
			CmnLog.error( this, "naming start exception", ex );
			throw( ex );
		} catch ( Throwable er ) {
			CmnLog.error( this, "naming start error", er );
		}
	}

	public void stopService() {
		naming.stop();
		CmnLog.debug(this,"JNP server stopped");
	}

	public void setJNPServerSocketFactory( String s ){
	}

	public void setInstallGlobalService(boolean var) {
	}

	public boolean getInstallGlobalService() {
		return false;
	}

	public void setRmiBindAddress(String addr) {
	}

	public String getRmiBindAddress() {
		return null;
	}

}
